# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


#' Subset the glance DataFrame based on selected variables.
#'
#' This function subsets the glance DataFrame to keep only the specified variables.
#'
#' @param glance_df The glance DataFrame to subset.
#' @param focus A character vector of variable names to keep, including "AIC", "BIC", "logLik", "deviance",
#' "df.resid", and "dispersion".
#' @return A subsetted glance DataFrame with only the selected variables.
#' @export
#'
#' @examples
#' data(iris)
#' models <-  fitModelParallel(Sepal.Length ~ Sepal.Width + Petal.Length, 
#'                        group_by = "Species",n.cores = 1, data = iris)
#' glance_df <- glance_tmb(models)
#' glance_df$group_id <- rownames(glance_df)
#' subset_glance(glance_df, c("AIC", "BIC"))
subset_glance <- function(glance_df, focus){
  idx_existing_column <- focus %in% c("AIC", "BIC", "logLik", "deviance" ,"df.resid", "dispersion" )
  if(sum(!idx_existing_column) > 0) warning(paste(focus[!idx_existing_column], ": does not exist\n"))
  focus <- focus[idx_existing_column]
  if (identical(focus, character(0)))
    stop(paste0("Please select at least one variable to focus on : ", 
                "AIC, BIC, logLik, deviance, df.resid, dispersion" ))
  glance_df <- glance_df[ , c("group_id", focus)]
  return(glance_df)
}


#' Plot Metrics for Generalized Linear Mixed Models (GLMM)
#'
#' This function generates a density plot of the specified metrics for the given
#' list of generalized linear mixed models (GLMMs).
#'
#' @param list_tmb A list of GLMM objects to extract metrics from.
#' @param focus A character vector specifying the metrics to focus on. Possible
#'   values include "AIC", "BIC", "logLik", "deviance", "df.resid", and
#'   "dispersion". If \code{NULL}, all available metrics will be plotted.
#'
#' @return A ggplot object displaying histogram plots for the specified metrics.
#'
#' @importFrom reshape2 melt
#' @importFrom ggplot2 aes facet_wrap geom_histogram theme_bw theme ggtitle
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' models_list <-  fitModelParallel(Sepal.Length ~ Sepal.Width + Petal.Length, 
#'                      group_by = "Species",n.cores = 1, data = iris)
#' diagnostic_plot(models_list, focus = c("AIC", "BIC", "deviance"))
diagnostic_plot <- function(list_tmb, focus = NULL) {
  glance_df <- glance_tmb(list_tmb)
  glance_df$group_id <- rownames(glance_df)
  if (!is.null(focus)) {
    glance_df <- subset_glance(glance_df, focus)
  }
  long_glance_df <- reshape2::melt(glance_df, variable.name = "metric")
  p <- ggplot2::ggplot(long_glance_df) +
    ggplot2::geom_histogram(ggplot2::aes(x = .data$value, col = .data$metric, fill = .data$metric)) +
    ggplot2::facet_wrap(~metric, scales = "free") +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = 'null') + 
    ggplot2::ggtitle("Metrics plot")
  return(p)
}


